package esac.archive.ehst.common.constants.client;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

/**
 * General constants used along EHST.
 */
public final class EhstGeneralConstants {

    /**
     * Other classes won't be able to extend this utility class with a private constructor.
     */
    private EhstGeneralConstants() {
        // not called
    }

    /**
     * Reg.expression used to validate target names. Starts with a letter, then it can contain any
     * word character (\w), a space, a dot, a comma, a minus or a plus sign, and ends with a word
     * character (\w). Note that \w is short for [a-zA-Z_0-9] Example of target names: CXO
     * J000002.0-094649 CXO J000002.0+094649 m 31
     */
    public static final String TARGET_NAME_VALID_PATTERN = "[a-zA-Z][-+ .\\w]*[\\w]";

    /**
     * HST Observation ID length IPPPSSOOT.
     */
    public static final int OBSERVATION_ID_LENGTH = 9;

    /** Unknown Position value. */
    public static final Double UNKNOWN_POSITION = -999.0;

    /** Unknown fov value. */
    public static final Double UNKNOWN_FOV = new Double(666);

    /** Coordinate reference system. */
    public static final String COORDINATE_SYSTEM_ICRS = "ICRS";

    /** Coordinate reference system. */
    public static final String COORDINATE_SYSTEM_FK5 = "FK5";

    /** Coordinate reference system. */
    public static final String COORDINATE_SYSTEM_FK4 = "FK4";

    /** Coordinate reference system. */
    public static final String COORDINATE_SYSTEM_GALACTIC = "GALACTIC";

    /** Coordinate reference system. */
    public static final String COORDINATE_SYSTEM_ECLIPTIC = "ECLIPTIC";

    /**
     * Instruments.
     */
    public enum InstrumentEnum {
        HRS("HRS"), STIS("STIS"), WFC3("WFC3"), FOS("FOS"), NICMOS("NICMOS"), WFPC2("WFPC2"), FOC(
                "FOC"), ACS("ACS"), COS("COS"), WFPC("WFPC");

        private static final Map<String, InstrumentEnum> instrumentMap = new HashMap<String, InstrumentEnum>();

        static {
            for (InstrumentEnum instrument : EnumSet.allOf(InstrumentEnum.class)) {
                instrumentMap.put(instrument.displayValue, instrument);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        InstrumentEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static InstrumentEnum get(String displayValue) {
            return instrumentMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }

    }

    /**
     * Calibration Level of each Plane.
     */
    public enum CalibrationLevelEnum {
        /** RAW Data. ObsCore Value: 1. */
        RAW("RAW"),
        /** CALIBRATED Data. ObsCore value: 2. */
        CALIBRATED("CALIBRATED"),
        /** Product Data. ObsCore value: 3. */
        PRODUCT("PRODUCT"),
        /** Auxiliary logs Data. */
        AUXILIARY("AUXILIARY");

        private static final Map<String, CalibrationLevelEnum> calibrationLevelMap = new HashMap<String, CalibrationLevelEnum>();

        static {
            for (CalibrationLevelEnum calibrationLevel : EnumSet.allOf(CalibrationLevelEnum.class)) {
                calibrationLevelMap.put(calibrationLevel.displayValue, calibrationLevel);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        CalibrationLevelEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static CalibrationLevelEnum get(String displayValue) {
            return calibrationLevelMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }
    }

    /**
     * Metadata Provenance Values.
     */
    public enum MetadataProvenanceEnum {
        /** Provenance values. */
        DADS("dads"), CACHE_META("cache/meta");

        private static final Map<String, MetadataProvenanceEnum> metadataProvenanceMap = new HashMap<String, MetadataProvenanceEnum>();

        static {
            for (MetadataProvenanceEnum metadataProvenance : EnumSet
                    .allOf(MetadataProvenanceEnum.class)) {
                metadataProvenanceMap.put(metadataProvenance.displayValue, metadataProvenance);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        MetadataProvenanceEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static MetadataProvenanceEnum get(String displayValue) {
            return metadataProvenanceMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }
    }

    /**
     * Data Product type of each Plane.
     */
    public enum DataProductTypeEnum {
        /** Spectrum. */
        TIME_RESOLVED_SPECTROSCOPY_1D("1D time resolved spectroscopy"),
        /** Spectrum. */
        SPECTROPOLARIMETRY_1D("1D spectropolarimetry"),
        /** Spectrum. */
        SPECTROSCOPY_1D("1D spectroscopy"),
        /** Spectrum. */
        SLIT_SPECTROSCOPY_2D("2D slit spectroscopy"),
        /** Spectrum. */
        SLITLESS_SPECTROSCOPY_2D("2D slitless spectroscopy"),
        /** Image. */
        IMAGING_CORONOGRAPHIC("Imaging coronagraphic"),
        /** Image. */
        IMAGING_POLARIMETRY("Imaging polarimetry"),
        /** Image. */
        IMAGING("Imaging"),
        /** Unknown Data product type. */
        UNKNOWN("Unknown");

        private static final Map<String, DataProductTypeEnum> dataProductTypeMap = new HashMap<String, DataProductTypeEnum>();

        static {
            for (DataProductTypeEnum dataProductType : EnumSet.allOf(DataProductTypeEnum.class)) {
                dataProductTypeMap.put(dataProductType.displayValue, dataProductType);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        DataProductTypeEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static DataProductTypeEnum get(String displayValue) {
            return dataProductTypeMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }
    }

    /**
     * Obs Core Data Product type of each Plane.
     */
    public enum ObsCoreDataProductTypeEnum {
        /** Spectrum. */
        SPECTRUM("spectrum"),
        /** Image. */
        IMAGE("image");

        private static final Map<String, ObsCoreDataProductTypeEnum> enumMap = new HashMap<String, ObsCoreDataProductTypeEnum>();

        static {
            for (ObsCoreDataProductTypeEnum obsCoreDataProductType : EnumSet
                    .allOf(ObsCoreDataProductTypeEnum.class)) {
                enumMap.put(obsCoreDataProductType.displayValue, obsCoreDataProductType);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        ObsCoreDataProductTypeEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static ObsCoreDataProductTypeEnum get(String displayValue) {
            return enumMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }
    }

    /**
     * Observation Intent types.
     */
    public enum ObservationIntentTypeEnum {
        /** Observation intent types. */
        SCIENCE("Science"), CALIBRATION("Calibration");

        private static final Map<String, ObservationIntentTypeEnum> observationIntentTypeMap = new HashMap<String, ObservationIntentTypeEnum>();

        static {
            for (ObservationIntentTypeEnum observationIntentType : EnumSet
                    .allOf(ObservationIntentTypeEnum.class)) {
                observationIntentTypeMap.put(observationIntentType.displayValue,
                        observationIntentType);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        ObservationIntentTypeEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static ObservationIntentTypeEnum get(String displayValue) {
            return observationIntentTypeMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }
    }

    /**
     * Ingestion Status.
     */
    public enum IngestionStatusEnum {
        /** IngestionStatus values. */
        OK("OK"), FAILED("FAILED");

        private static final Map<String, IngestionStatusEnum> ingestionStatusMap = new HashMap<String, IngestionStatusEnum>();

        static {
            for (IngestionStatusEnum ingestionStatus : EnumSet.allOf(IngestionStatusEnum.class)) {
                ingestionStatusMap.put(ingestionStatus.displayValue, ingestionStatus);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        IngestionStatusEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static IngestionStatusEnum get(String displayValue) {
            return ingestionStatusMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }
    }

    /**
     * Types of ingestion.
     */
    public enum IngestionTypeEnum {
        /** Ingestion types. */
        HST_COMPOSITE_OBSERVATION("HST Composite Observation"), PUBLICATION("Publication");

        private static final Map<String, IngestionTypeEnum> ingestionTypeMap = new HashMap<String, IngestionTypeEnum>();

        static {
            for (IngestionTypeEnum ingestionType : EnumSet.allOf(IngestionTypeEnum.class)) {
                ingestionTypeMap.put(ingestionType.displayValue, ingestionType);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        IngestionTypeEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static IngestionTypeEnum get(String displayValue) {
            return ingestionTypeMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }
    }

    /**
     * Optical Element types.
     */
    public enum OpticalElementTypeEnum {
        /** Optical Element types. */
        GRISM("Grism"), GRATING("Grating"), ECHELLE("Echelle"), PRISM("Prism"), NARROWBAND(
                "Narrowband"), MEDIUMBAND("Mediumband"), BROADBAND("Broadband"), NEUTRALDENSITY(
                        "Neutral density"), UNKNOWN("Unknown");

        private static final Map<String, OpticalElementTypeEnum> opticalElementTypeMap = new HashMap<String, OpticalElementTypeEnum>();

        static {
            for (OpticalElementTypeEnum opticalElementType : EnumSet
                    .allOf(OpticalElementTypeEnum.class)) {
                opticalElementTypeMap.put(opticalElementType.displayValue, opticalElementType);
            }
        }
        /** Display value on the Web. */
        private final String displayValue;

        /**
         * Constructor.
         * @param displayValue
         */
        OpticalElementTypeEnum(String displayValue) {
            this.displayValue = displayValue;
        }

        public String getDisplayValue() {
            return this.displayValue;
        }

        public static OpticalElementTypeEnum get(String displayValue) {
            return opticalElementTypeMap.get(displayValue);
        }

        @Override
        public String toString() {
            return this.getDisplayValue();
        }
    }

    /**
     * Haplpha, U, B, V, R, I, J, Jbroad, H, K, L, M, HeI, HeI-C, FeII, FeII-C, Paschen-Alpha,
     * Paschen-Alpha-C, Millimeter, Infrared, Optical, Ultraviolet
     */
    public enum BandEnum {
        /* SCIENCE */
        HALPHA("Halpha", 656.2, 2.85), U("U", 365, 68), B("B", 440, 98), V("V", 550, 89), R("R",
                700, 220), I("I", 900, 240), J("J", 1250, 300), JBROAD("JBroad", 1250.0, 400), H(
                        "H", 1650, 400), K("K", 2200, 600), L("L", 3600, 1200), M("M", 4800, 800), HEI(
                                "HeI", 1080.8, 10.1), HEI_C("HeI-C", 1129.1, 11.4), FEII("FeII", 1646.0, 17.3), FEII_C(
                                        "FeII-C", 1659.5, 17), PASCHEN_ALPHA("Paschen-Alpha", 1874.5, 19.5), PASCHEN_ALPHA_C(
                                                "Paschen-Alpha-C", 1899.8, 17.9), MILLIMETER("Millimeter", 0.505E+7, 0.99E+7), INFRARED(
                                                        "Infrared", 0.505E+5, 0.99E+5), OPTICAL("Optical", 650., 700.), ULTRAVIOLET(
                                                                "Ultraviolet", 155, 290);

        private final String bandname;
        private final double wavelength; // in nm
        private final double wavefwhm; // in nm

        BandEnum(String name, double wavelength, double wavefwhm) {
            this.bandname = name;
            this.wavelength = wavelength;
            this.wavefwhm = wavefwhm;
        }

        public String bandname() {
            return bandname;
        }

        public double wavelength() {
            return wavelength;
        }

        public double wavefwhm() {
            return wavefwhm;
        }
    }

}
